﻿using System;
using System.IO;
using System.Linq;
using System.Text;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Drawing.Drawing2D;
using System.Collections.Generic;

using JsDrawControl;
using JsDrawControl.Item;
using JsDrawControl.Panel;
using JsDrawControl.History;
using JsDrawControl.ShortCut;

namespace JsDraw
{
    public partial class MainForm : Form
    {
        #region メンバ変数

        /// <summary>イベント処理を行うかどうかを制御します。</summary>
        /// <remarks>true:イベント処理を行う、false:イベント処理を行わない</remarks>
        private bool eventFlag;

        /// <summary>ロードファイル名</summary>
        private string loadFileName;

        /// <summary>ロードファイルフォーマット</summary>
        private FormatType loadFileFormat;

        /// <summary>ファイル履歴</summary>
        private ToolStripMenuItem[] fileHistory = null;

        /// <summary>プロパティ変更開始フラグ</summary>
        /// <remarks>プロパティ変更が開始されている場合は true を設定</remarks>
        private bool chgPropertyStartFlag;

        #endregion

        #region コンストラクタ

        //=====================================================================
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            // メンバ変数初期化
            this.eventFlag = false;
            this.loadFileName = string.Empty;
            this.loadFileFormat = FormatType.xml;

            // ヒストリメニュー作成
            this.fileHistory = new ToolStripMenuItem[5];
            this.fileHistory[0] = this.menuFileHistory1;
            this.fileHistory[1] = this.menuFileHistory2;
            this.fileHistory[2] = this.menuFileHistory3;
            this.fileHistory[3] = this.menuFileHistory4;
            this.fileHistory[4] = this.menuFileHistory5;

            this.chgPropertyStartFlag = false;
        }

        #endregion

        #region フォームロードイベント

        //=====================================================================
        /// <summary>
        /// フォームロードイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // 画面項目クリア
            this.statusMousePoint.Text = string.Empty;
            this.statusSelectBounds.Text = string.Empty;
            this.statuslPaperSize.Text = string.Empty;

            // タイトル設定
            this.Text = Program.PROGRAM_TITLE;

            // イベント処理を行うようにフラグを設定
            this.eventFlag = true;

            // 新しいドキュメントを作成
            this.NewDoc();

            // ファイル履歴メニュー設定
            this.SetFileHistMenu();

            // ボタン設定
            this.toolItemDefault.Checked = true;
        }

        #endregion

        #region フォームクローズイベント

        //=====================================================================
        /// <summary>
        /// フォームクローズ確認イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 内容が変更されている場合
            if (this.drawView.IsChanged)
            {
                // 保存確認
                DialogResult result = MessageBox.Show(
                    this,
                    MessageDef.GetMessage(MessageDef.I001001),
                    Program.PROGRAM_TITLE,
                    MessageBoxButtons.YesNoCancel,
                    MessageBoxIcon.Question);

                switch( result)
                {
                    case DialogResult.Yes:
                        // 保存に失敗した場合、クローズキャンセル
                        if (!this.Save())                           
                            e.Cancel = true;
                        break;

                    case DialogResult.Cancel:
                        // クローズキャンセル
                        e.Cancel = true;
                        break;
                }
            }
        }

        //=====================================================================
        /// <summary>
        /// フォームクローズイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            // ========== 画面状態の保存 ==========
            // メインスプリッタの距離
            Program.Config.MainSplitterDistance = this.splitMain.SplitterDistance;

            // アイテムスプリッタの距離
            Program.Config.ItemSplitterDistance = this.splitItem.SplitterDistance;

            // 画面位置
            Program.Config.WindowRectangle =
                new Rectangle(
                    this.Location.X, this.Location.Y,
                    this.Size.Width, this.Size.Height);

            // 画面状態
            Program.Config.WindowState = this.WindowState;
        }

        #endregion

        #region 「新規作成」ボタンクリックイベント
        
        //=====================================================================
        /// <summary>
        /// 「新規作成」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonNew_Click(object sender, EventArgs e)
        {
            // 新規作成
            this.NewDoc(); 
        }

        #endregion

        #region 「開く」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「開く」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonOpen_Click(object sender, EventArgs e)
        {
            // 読み込み処理
            this.Read();
        }

        #endregion

        #region 「保存」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「保存」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSave_Click(object sender, EventArgs e)
        {
            // 上書き保存
            this.Save();
        }

        #endregion

        #region 「設定」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「設定」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSetting_Click(object sender, EventArgs e)
        {
            SettingForm setting = new SettingForm();
            setting.Config = Program.Config;

            // 設定画面表示
            if (setting.ShowDialog(this) == DialogResult.OK)
            {
                Program.Config = setting.Config;
                this.SettingForm(Program.Config, false);
            }
        }

        #endregion

        #region 「プレビュー」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「プレビュー」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPreview_Click(object sender, EventArgs e)
        {
            MessageBox.Show(this, "プレビュー表示はまだ実装されていません。", 
                Program.PROGRAM_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        #endregion

        #region 「印刷」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「印刷」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPrint_Click(object sender, EventArgs e)
        {
            MessageBox.Show(this, "印刷処理はまだ実装されていません。",
                Program.PROGRAM_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        #endregion

        #region 「カーソル」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「カーソル」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void toolItemDefault_Click(object sender, EventArgs e)
        {
            // デフォルトモード設定
            this.drawView.ControlMode = ControlMode.Default;
            this.toolItemDefault.Checked = true;
        }

        #endregion

        #region 「ビュー」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「ビュー」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonView_Click(object sender, EventArgs e)
        {
            // デフォルトモード設定
            this.drawView.ControlMode = ControlMode.View;
            this.toolItemView.Checked = true;
        }

        #endregion

        #region 「移動」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「移動」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonHand_Click(object sender, EventArgs e)
        {
            // ハンドモード設定
            this.drawView.ControlMode = ControlMode.Hnad;
            this.toolItemHand.Checked = true;
        }

        #endregion

        #region 「ズーム」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「ズーム」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonZoom_Click(object sender, EventArgs e)
        {
            // ハンドモード設定
            this.drawView.ControlMode = ControlMode.Zoom;
            this.toolItemZoom.Checked = true;
        }

        #endregion

        #region 「アイテム選択」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「アイテム選択」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemSelect_Click(object sender, EventArgs e)
        {
            // ハンドモード設定
            this.drawView.ControlMode = ControlMode.Select;
            this.toolItemSelect.Checked = true;
        }

        #endregion

        #region 「消しゴム」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「消しゴム」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void toolErase_Click(object sender, EventArgs e)
        {
            // 消しゴムモード設定
            this.drawView.ControlMode = ControlMode.Erase;
            this.toolErase.Checked = true;
        }

        #endregion

        #region 「縮小」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「縮小」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonZoomOut_Click(object sender, EventArgs e)
        {
            // ズームコンボボックス設定
            int index = this.toolZoom.SelectedIndex - 1;

            if (0 <= index && index < this.toolZoom.Items.Count)
                this.toolZoom.SelectedIndex = index;
        }

        #endregion

        #region 「拡大」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「拡大」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonZoomIn_Click(object sender, EventArgs e)
        {
            // ズームコンボボックス設定
            int index = this.toolZoom.SelectedIndex + 1;

            if (0 <= index && index < this.toolZoom.Items.Count)
                this.toolZoom.SelectedIndex = index;
        }

        #endregion

        #region 「元に戻す」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「元に戻す」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonUndo_Click(object sender, EventArgs e)
        {
            // 元に戻す
            if (this.drawView.CanUndo())
                this.drawView.Undo();
        }

        #endregion

        #region 「やり直し」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「やり直し」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRedo_Click(object sender, EventArgs e)
        {
            // やり直し
            if (this.drawView.CanRedo())
                this.drawView.ReDo();
        }

        #endregion

        #region 「削除」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「削除」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonDelete_Click(object sender, EventArgs e)
        {
            // 選択されているアイテムを削除
            this.drawView.RemoveSelectedItem();
        }

        #endregion

        #region 「前面に移動」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「前面に移動」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonOrderUp_Click(object sender, EventArgs e)
        {
            // 選択されているアイテムを移動
            this.drawView.MoveItemOrder(ItemOrderDirection.Up);
        }

        #endregion

        #region 「背面に移動」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「背面に移動」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonOrderDown_Click(object sender, EventArgs e)
        {
            // 選択されているアイテムを移動
            this.drawView.MoveItemOrder(ItemOrderDirection.Down);
        }

        #endregion

        #region 「切り取り」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「切り取り」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCut_Click(object sender, EventArgs e)
        {
            // 選択されているアイテムを切り取り
            this.drawView.CutItem();
        }
        
        #endregion

        #region 「コピー」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「コピー」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCopy_Click(object sender, EventArgs e)
        {
            // 選択されているアイテムをコピー
            this.drawView.CopyItem();
        }
        
        #endregion

        #region 「貼り付け」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「貼り付け」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPaste_Click(object sender, EventArgs e)
        {
            // 選択されているアイテムを貼り付け
            this.drawView.PasteItem();
        }

        #endregion

        #region 「名前を付けて保存」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「名前を付けて保存」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSaveAs_Click(object sender, EventArgs e)
        {
            // 名前をつけて保存
            this.SaveAs();
        }

        #endregion

        #region 「ファイル履歴」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「ファイル履歴」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonFileHistory_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem item = (ToolStripMenuItem)sender;

            // ファイル名取得
            string fileName = (string)item.Tag;

            // ファイル存在チェック
            if (!File.Exists(fileName))
            {
                // エラーメッセージ表示
                MessageBox.Show(
                    this,
                    MessageDef.GetMessage(
                        MessageDef.E001003,
                        fileName),
                    Program.PROGRAM_TITLE,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Warning);
                return;
            }

            // ファイル読み込み
            this.Read(fileName);
        }

        #endregion

        #region 「終了」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「終了」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonEnd_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        #endregion

        #region 「ヘルプ」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「ヘルプ」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonHelpFile_Click(object sender, EventArgs e)
        {
            MessageBox.Show(this, "ヘルプはまだ作成していません。",
                Program.PROGRAM_TITLE, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        #endregion

        #region 「バージョン」ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// 「バージョン」ボタンクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonVersion_Click(object sender, EventArgs e)
        {
            VersionForm ver = new VersionForm();
            ver.ShowDialog(this);
        }

        #endregion

        #region [操作履歴クリア]ボタンクリックイベント

        //=====================================================================
        /// <summary>
        /// [操作履歴クリア]ボタンクリックイベント処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonClearHistory_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(this, "操作履歴をクリアしてもよろしいですか？", 
                Program.PROGRAM_TITLE, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                return;

            // コマンド履歴クリア
            this.drawView.CommandHistoryClear();

            // ヒストリリスト更新
            this.SetHistoryList();
        }
        #endregion

        #region アイテムツールボタンイベント処理

        //=====================================================================
        /// <summary>
        /// ラインアイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemLine_Click(object sender, EventArgs e)
        {
            this.UnCheckItemButton();

            // 直線アイテム作成
            LineItem item = new LineItem();
            item.Name = "直線";

            // アイテムをデザインモードで追加
            this.drawView.AddDrawItem(item, true);
            this.toolItemLine.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// 曲線アイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemPolyLine_Click(object sender, EventArgs e)
        {
            this.UnCheckItemButton();

            // 曲線アイテム作成
            PolyLineItem item = new PolyLineItem();
            item.Name = "曲線";

            // アイテムをデザインモードで追加
            this.drawView.AddDrawItem(item, true);
            this.toolItemPolyLine.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// フリーアイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemFreeLine_Click(object sender, EventArgs e)
        {
            this.UnCheckItemButton();

            // フリーアイテム作成
            FreeLineItem item = new FreeLineItem();
            item.Name = "ペン";

            // アイテムをデザインモードで追加
            this.drawView.AddDrawItem(item, true);
            this.toolItemFreeLine.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// 円アイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemCircle_Click(object sender, EventArgs e)
        {
            this.UnCheckItemButton();

            // 円アイテム作成
            EllipseItem item = new EllipseItem();
            item.Name = "円";

            // アイテムをデザインモードで追加
            this.drawView.AddDrawItem(item, true);
            this.toolItemCircle.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// 三角アイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemTriangle_Click(object sender, EventArgs e)
        {
            this.UnCheckItemButton();

            // 三角形アイテム作成
            TrianguleItem item = new TrianguleItem();
            item.Name = "三角形";

            // アイテムをデザインモードで追加
            this.drawView.AddDrawItem(item, true);
            this.toolItemTriangle.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// 四角アイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemRectangle_Click(object sender, EventArgs e)
        {
            this.UnCheckItemButton();

            // 四角アイテム作成
            RectItem item = new RectItem();
            item.Name = "四角形";

            // アイテムをデザインモードで追加
            this.drawView.AddDrawItem(item, true);
            this.toolItemRectangle.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// ラウンドアイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemRound_Click(object sender, EventArgs e)
        {
            this.UnCheckItemButton();

            // ラウンドアイテム作成
            RoundItem item = new RoundItem();
            item.Name = "ラウンド";

            // アイテムをデザインモードで追加
            this.drawView.AddDrawItem(item, true);
            this.toolItemRound.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// 多角形アイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemPolygon_Click(object sender, EventArgs e)
        {
            this.UnCheckItemButton();

            // 多角形アイテム作成
            PolygonItem item = new PolygonItem();
            item.Name = "多角形";

            // アイテムをデザインモードで追加
            this.drawView.AddDrawItem(item, true);
            this.toolItemPolygon.Checked = true;
        }

        //=====================================================================
        /// <summary>
        /// グループアイテム配置イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonItemGroup_Click(object sender, EventArgs e)
        {
            this.UnCheckItemButton();

            // イメージアイテム作成
            GroupItem item = new GroupItem();
            item.Name = "グループ";

            // アイテムをデザインモードで追加
            this.drawView.AddDrawItem(item, true);
            this.toolItemGroup.Checked = true;
        }
        
        #endregion

        #region [J.S Drawコントロール]　マウス移動イベント

        //=====================================================================
        /// <summary>
        /// マウス移動イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_MouseMove(object sender, MouseEventArgs e)
        {
            // マウス位置をスケール座標に変換する。
            PointF scrPoint = this.drawView.GetScalePoint(e.Location);

            this.statusMousePoint.Text =
                string.Format(" X:{0:0.0:}, Y:{1:0.0} (mm)",
                scrPoint.X, scrPoint.Y);

            if (this.drawView.SpinAngle != 0.0F)
            {
                // ※回転角度プロパティが設定されている場合はアイテム回転中

                // イメージ設定
                this.statusSelectBounds.Image = this.imageStatus.Images[2];

                // テキスト設定
                this.statusSelectBounds.Text =
                    string.Format(" {0:0.00}°", this.drawView.SpinAngle);
            }
            else if (this.drawView.MouseFrameRect != RectangleF.Empty)
            {
                // ※マウス選択範囲プロパティが設定されている場合はマウスで範囲選択中

                // イメージ設定
                this.statusSelectBounds.Image = this.imageStatus.Images[0];

                // テキスト設定
                this.statusSelectBounds.Text =
                    string.Format(" X:{0:0.0}, Y:{1:0.0} (mm) - W:{2:0.0}, H:{3:0.0} (mm)",
                    this.drawView.MouseFrameRect.X,
                    this.drawView.MouseFrameRect.Y,
                    this.drawView.MouseFrameRect.Width,
                    this.drawView.MouseFrameRect.Height);
            }
            else if (this.drawView.SelectedItemBounds != RectangleF.Empty)
            {
                // ※選択アイテム範囲が設定されている場合はアイテムが選択されている。

                // イメージ設定
                this.statusSelectBounds.Image = this.imageStatus.Images[1];

                // テキスト設定
                this.statusSelectBounds.Text =
                    string.Format(" X:{0:0.0}, Y:{1:0.0} (mm) - W:{2:0.0}, H:{3:0.0} (mm)",
                    this.drawView.SelectedItemBounds.X,
                    this.drawView.SelectedItemBounds.Y,
                    this.drawView.SelectedItemBounds.Width,
                    this.drawView.SelectedItemBounds.Height);
            }
            else
            {
                // イメージ設定
                this.statusSelectBounds.Image = this.imageStatus.Images[0];
                // テキスト設定
                this.statusSelectBounds.Text = string.Empty;
            }
        }

        #endregion

        #region [J.S Drawコントロール]　コントロールモード変更イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　コントロールモード変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_ControlModeChanged(
            object sender, JsDrawControl.ControlModeChangedEventArgs e)
        {
            // ボタンチェック状態
            bool chkDef = false;
            bool chkHand = false;
            bool chkZoom = false;
            bool chkSelect = false;
            bool chkView = false;
            bool chkErase = false;
            bool chkTool = false;

            // コントロールモードボタン設定
            switch (e.ControlMode)
            {
                case ControlMode.Default:   // デフォルトモード
                case ControlMode.ItemEdit:  // アイテム編集モード
                    chkDef = true;
                    break;

                case ControlMode.Hnad:      // ハンドモード
                    chkHand = true;
                    break;

                case ControlMode.Zoom:      // ズームモード
                    chkZoom = true;
                    break;

                case ControlMode.Select:    // 選択モード
                    chkSelect = true;
                    break;

                case ControlMode.View:      // ビューモード
                    chkView = true;
                    break;

                case ControlMode.Erase:     // 消しゴムモード
                    chkErase = true;
                    break;

                case ControlMode.ItemAdd:   // アイテム追加モード
                    chkTool = true;
                    break;
            }

            // ========== ボタン状態設定 ==========
            // デフォルト
            this.toolItemDefault.Checked = chkDef;

            // 移動
            this.toolItemHand.Checked = chkHand;

            // ズーム
            this.toolItemZoom.Checked = chkZoom;

            // 選択範囲
            this.toolItemSelect.Checked = chkSelect;

            // ビュー
            this.toolItemView.Checked = chkView;

            // 消しゴム
            this.toolErase.Checked = chkErase;

            // ツールバーのボタンを戻す
            if (chkTool == false)
            {
                this.toolItemLine.Checked = false;
                this.toolItemPolyLine.Checked = false;
                this.toolItemFreeLine.Checked = false;
                this.toolItemCircle.Checked = false;
                this.toolItemTriangle.Checked = false;
                this.toolItemRectangle.Checked = false;
                this.toolItemRound.Checked = false;
                this.toolItemPolygon.Checked = false;
            }
        }

        #endregion

        #region [J.S Drawコントロール]　描画アイテム追加イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　描画アイテム追加イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_DrawItemInsert(object sender, JsDrawControl.DrawItemInsertEventArgs e)
        {
            // アイテムリスト更新
            this.SetItemList();

            // ヒストリリスト更新
            this.SetHistoryList();
        }

        #endregion

        #region [J.S Drawコントロール]　描画アイテム追加キャンセルイベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　描画アイテム追加キャンセルイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_CancelDrawItemInsert(object sender, JsDrawControl.CancelDrawItemInsertEventArgs e)
        {
            // アイテムツールボタンのチェックを外す
            this.UnCheckItemButton();
        }

        #endregion

        #region [J.S Drawコントロール]　ズーム率変更イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　ズーム率変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_ZoomChangedEvent(
            object sender, JsDrawControl.ZoomChangedEventArgs e)
        {
            try
            {
                // コンボボックスのイベントを発生させなくするため、イベントフラグを false に設定する
                this.eventFlag = false;

                // ズーム設定
                this.SetZoom(e.Zoom);
            }
            finally
            {
                // イベントフラグを戻す
                this.eventFlag = true;
            }
        }

        #endregion

        #region [J.S Drawコントロール]　選択描画アイテム変更イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　選択描画アイテム変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_DrawItemSelectedChanged(
            object sender, JsDrawControl.DrawItemSelectedChangedEventArgs e)
        {
            // ユーザーによって行われた操作の場合
            if (e.Action == DrawItemAction.ByUser)
            {
                if (e.Items == null || e.Items.Length == 0)
                {
                    this.propertyItem.SelectedObject = this.drawView.Panel;
                }
                else
                {
                    this.propertyItem.SelectedObjects = e.Items;
                }

                // アイテムリスト更新
                this.UpdateItemListSelect();
            }
        }

        #endregion

        #region [J.S Drawコントロール]　描画アイテム変更イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　描画アイテム変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_DrawItemChanged(
            object sender, JsDrawControl.DrawItemChangedEventArgs e)
        {
            // ユーザーによって行われた操作の場合
            if (e.Action == DrawItemAction.ByUser)
            {
                // アイテムリスト更新
                this.UpdateItemList();

                // プロパティグリッド更新
                this.UpdatePropertyGrid();

                // 履歴リスト設定
                this.SetHistoryList();
            }
            else
            {
                // ユーザー操作でない場合はプロパティ変更フラグを true に設定する
                // 複数アイテムを選択した状態でプロパティグリッドの値を変更すると描画に時間がかかるため、
                // "chgPropertyStartFlag" フラグを使って一括で描画が行えるようにします。
                if (this.chgPropertyStartFlag == false)
                {
                    this.chgPropertyStartFlag = true;
                    this.drawView.BeginInit();
                }
            }
        }

        #endregion

        #region [J.S Drawコントロール]　描画アイテムリムーブイベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　描画アイテムリムーブイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_DrawItemRemove(
            object sender, JsDrawControl.DrawItemRemoveEventArgs e)
        {
            // アイテムリスト更新
            this.SetItemList();

            // プロパティグリッド更新
            this.UpdatePropertyGrid();

            // ヒストリリスト更新
            this.SetHistoryList();
        }

        #endregion

        #region [J.S Drawコントロール]　描画パネル変更イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　描画パネル変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_PanelItemChanged(
            object sender, JsDrawControl.PanelChangedEventArgs e)
        {
            // ステータスバー設定
            this.SetPaperSizeStatus();

            // プロパティグリッド更新
            this.UpdatePropertyGrid();

            // ヒストリリスト更新
            this.SetHistoryList();
        }

        #endregion

        #region [J.S Drawコントロール]　描画アイテムオーダー変更イベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　描画アイテムオーダー変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_DrawItemChangeOrder(
            object sender, JsDrawControl.DrawItemChangeOrderEventArgs e)
        {
            // アイテムリスト更新
            this.SetItemList();

            // ヒストリリスト更新
            this.SetHistoryList();
        }

        #endregion

        #region [J.S Drawコントロール]　やり直しイベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　やり直しイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_ReDoing(
            object sender, JsDrawControl.ReDoingEventArgs e)
        {
            // アイテムリスト更新
            this.SetItemList();

            // ヒストリリスト更新
            this.SetHistoryList();
        }

        #endregion

        #region [J.S Drawコントロール]　元に戻すイベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　元に戻すイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_UnDoing(
            object sender, JsDrawControl.UnDoingEventArgs e)
        {
            // アイテムリスト更新
            this.SetItemList();

            // ヒストリリスト更新
            this.SetHistoryList();
        }

        #endregion

        #region [J.S Drawコントロール]　マウスクリックイベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　マウスクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_MouseClick(object sender, MouseEventArgs e)
        {

        }

        #endregion

        #region [J.S Drawコントロール]　マウスダブルクリックイベント

        //=====================================================================
        /// <summary>
        /// [J.S Drawコントロール]　マウスダブルクリックイベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void drawView_MouseDoubleClick(object sender, MouseEventArgs e)
        {

        }

        #endregion

        #region [アイテムリスト]　選択インデックス変更イベント

        //=====================================================================
        /// <summary>
        /// [アイテムリスト]　選択インデックス変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listItem_SelectedIndexChanged(object sender, EventArgs e)
        {
            // イベントを処理しない場合は終了
            if (this.eventFlag == false)
                return;
            
            try
            {
                this.drawView.BeginInit();

                // アイテム選択クリア
                this.drawView.SelectedItems.Clear();

                foreach (ListViewItem listItem in this.listItem.Items)
                {
                    // アイテムリストで選択中のアイテムを DrawView でも選択する。
                    if (listItem.Selected)
                        this.drawView.SelectedItems.Add(listItem.Tag as IDrawItem);
                }

                // プロパティグリッド設定
                if (0 < this.drawView.SelectedItems.Count)
                {
                    this.propertyItem.SelectedObjects = this.drawView.SelectedItems.ToArray();
                }
                else
                {
                    this.propertyItem.SelectedObject = this.drawView.Panel;
                }
            }
            finally
            {
                this.drawView.EndInit();
            }
        }

        #endregion

        #region [プロパティグリッド]　プロパティ値変更イベント

        //=====================================================================
        /// <summary>
        /// [プロパティグリッド]　プロパティ値変更イベント処理を行います。
        /// </summary>
        /// <param name="s"></param>
        /// <param name="e"></param>
        private void propertyItem_PropertyValueChanged(
            object s, PropertyValueChangedEventArgs e)
        {
            // 複数アイテムを選択した状態でプロパティグリッドの値を変更すると描画に時間がかかるため、
            // "chgPropertyStartFlag" フラグを使って一括で描画が行えるようにします。
            if (this.chgPropertyStartFlag)
            {
                this.chgPropertyStartFlag = false;
                this.drawView.EndInit();

                // アイテムリスト更新
                this.SetItemList();

                // プロパティグリッド更新
                this.UpdatePropertyGrid();

                // ヒストリリスト更新
                this.SetHistoryList();
            }
        }

        #endregion

        #region [ズームコンボボックス]　選択インデックス変更イベント

        //=====================================================================
        /// <summary>
        /// ズームコンボボックス選択インデックス変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void toolZoom_SelectedIndexChanged(object sender, EventArgs e)
        {
            // イベントを処理しない場合は終了
            if (this.eventFlag == false)
                return;

            if (this.toolZoom.SelectedItem != null)
            {
                ListItem item = (ListItem)this.toolZoom.SelectedItem;

                // 拡大率の設定
                this.drawView.Zoom = item.FloatValue;
            }
        }

        #endregion

        #region 操作タブイベント

        //=====================================================================
        /// <summary>
        /// 操作タブインデックス変更イベント処理を行います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tabOperation_SelectedIndexChanged(object sender, EventArgs e)
        {
            // アイテムリスト設定
            this.SetItemList();

            // 操作履歴リスト設定
            this.SetHistoryList();
        }

        #endregion

        #region プライベートメソッド

        //=====================================================================
        /// <summary>
        /// ツールバーのアイテムボタンのチェックをすべて外します。
        /// </summary>
        private void UnCheckItemButton()
        {
            // ツールバーのボタンを戻す
            foreach (ToolStripItem item in this.toolItem.Items)
            {
                if (item is ToolStripButton)
                {
                    var btn = item as ToolStripButton;

                    if (btn.Checked)
                        btn.Checked = false;
                }
            }
        }

        //=====================================================================
        /// <summary>
        /// ツールバーのズームボタンの設定を行います。
        /// </summary>
        /// <param name="zoom"></param>
        private void SetZoom(float zoom)
        {
            // アイテム検索
            int index = -1;
            for (int i = 0; i < this.toolZoom.Items.Count; i++)
            {
                index = i;
                ListItem item = (ListItem)this.toolZoom.Items[i];
                if (zoom <= item.FloatValue)
                    break;
            }

            // コンボボックス設定
            if (this.toolZoom.SelectedIndex != index)
                this.toolZoom.SelectedIndex = index;

            // ========== 拡大、縮小ボタン設定 ==========
            // 縮小ボタン設定
            if (0 < this.toolZoom.SelectedIndex)
            {
                this.toolZoomOut.Enabled = true;
            }
            else
            {
                this.toolZoomOut.Enabled = false;
            }

            // 拡大ボタン設定
            if (this.toolZoom.SelectedIndex < this.toolZoom.Items.Count - 1)
            {
                this.toolZoomIn.Enabled = true;
            }
            else
            {
                this.toolZoomIn.Enabled = false;
            }
        }

        //=====================================================================
        /// <summary>
        /// 用紙サイズステータスバーの設定を行います。
        /// </summary>
        private void SetPaperSizeStatus()
        {
            this.statuslPaperSize.Text =
                string.Format(" 用紙サイズ：横 {0}mm × 縦 {1}mm",
                this.drawView.Panel.PanelSize.Width,
                this.drawView.Panel.PanelSize.Height);
        }

        //=====================================================================
        /// <summary>
        /// タイトル文字列の設定を行います。
        /// </summary>
        private void SetTitle()
        {
            string fileName = string.Empty;

            if (loadFileName == string.Empty)
            {
                fileName = "新規";
            }
            else
            {
                fileName = Path.GetFileNameWithoutExtension(loadFileName);
            }

            this.Text = string.Format("{0} [{1}]", Program.PROGRAM_TITLE, fileName);
        }

        //=====================================================================
        /// <summary>
        /// ファイル履歴メニューの設定を行います。
        /// </summary>
        private void SetFileHistMenu()
        {
            for (int i = 1; i <= 5; i++)
            {
                int index = i - 1;

                if (i <= Program.Config.FileHistory.Count)
                {
                    fileHistory[index].Visible = true;
                    fileHistory[index].Text =
                        string.Format("&{0} {1}", i, Program.Config.FileHistory[index]);
                    fileHistory[index].Tag = Program.Config.FileHistory[index];
                }
                else
                {
                    fileHistory[index].Visible = false;
                }
            }
        }

        #endregion

        #region フォーム設定処理

        //=====================================================================
        /// <summary>
        /// フォームを設定します。
        /// </summary>
        /// <param name="config">設定</param>
        /// <param name="setFormStatus">フォームステータスを設定</param>
        public void SettingForm(ConfigInfo config, bool setFormStatus)
        {
            try
            {
                // イベント処理を行うようにフラグを設定
                this.eventFlag = false;

                // ========== ズームコンボボックス設定処理 ==========
                // 設定前の値を取得する
                float befValue = 1.0F;
                if (this.toolZoom.SelectedItem != null)
                    befValue = (this.toolZoom.SelectedItem as ListItem).FloatValue;

                this.toolZoom.Items.Clear();

                // コンボボックス設定
                int minZoom = (int)(config.MinZoom * 100F);
                int maxZoom = (int)(config.MaxZoom * 100F);
                for(int z= minZoom ; z <= maxZoom ; z += 5)
                    this.toolZoom.Items.Add(new ListItem(string.Format("{0}%", z), (float)z / 100F));

                // 値の範囲確認
                if (befValue < config.MinZoom)befValue = config.MinZoom;
                if (befValue > config.MaxZoom)befValue = config.MaxZoom;

                // コンボボックス選択
                SetZoom(befValue);

                // DrawView 設定
                this.drawView.ImageQuarity = config.ImageQuarity;
                this.drawView.SmoothingMode = config.SmoothingMode;
                this.drawView.PanelOffset = config.PanelOffset;               
                this.drawView.CommnadHistoryCount = config.HistoryCount;
                this.drawView.ScreenBufferSize = config.ScreenBufferSize;                
                this.drawView.AdjusterMoveStyle = config.AdjusterMoveStyle;
                this.drawView.ShowVScrollBar = config.ShowVScrollBar;
                this.drawView.ShowHScrollBar = config.ShowHScrollBar;
                this.drawView.ShowRuler = config.ShowRuler;
                this.drawView.CornerText = config.CornerText;
                this.drawView.MinZoom = config.MinZoom;
                this.drawView.MaxZoom = config.MaxZoom;
                this.drawView.ClipBoardDataFormat = config.ClipBoardDataFormat;
                this.drawView.ClipBoardImageFormatSize = config.ClipBoardImageFormatSize;
                this.drawView.UsePenTab = config.UsePentab;
                this.drawView.EraserSize = config.EraserSize;

                // ショートカット設定
                if (0 == config.ShortCut.CommandKeyList.Count)
                {
                    config.ShortCut = this.drawView.ShortCut.Clone() as ShortCutManager;
                }
                else
                {
                    this.drawView.ShortCut.CommandKeyList = config.ShortCut.CommandKeyList;
                }
                this.drawView.ShortCut.Enabled = config.ShortCut.Enabled;
                this.drawView.MultiSelect = config.MultiSelect;
                
                // ステータスバー設定
                this.SetPaperSizeStatus();

                if (setFormStatus)
                {
                    // ========== メイン画面設定 ==========
                    // ウインドウ内に収まるかチェックする
                    bool sizeChkFlag = false;

                    foreach (Screen sc in Screen.AllScreens)
                    {
                        if (sc.WorkingArea.Contains(config.WindowRectangle))
                        {
                            sizeChkFlag = true;
                            break;
                        }
                    }

                    // ウインドウに収まらない場合は初期値
                    if (sizeChkFlag == false)
                        config.WindowRectangle = new Rectangle(100, 100, 800, 600);

                    // ウィンドウロケーション
                    this.Location = new Point(config.WindowRectangle.X, config.WindowRectangle.Y);

                    // ウィンドウサイズ
                    this.Size = new Size(config.WindowRectangle.Width, config.WindowRectangle.Height);

                    // ウインドウステート
                    if (config.WindowState == FormWindowState.Minimized)
                    {
                        this.WindowState = FormWindowState.Normal;
                    }
                    else
                    {
                        this.WindowState = config.WindowState;
                    }

                    // メインスプリッタの距離
                    this.splitMain.SplitterDistance = config.MainSplitterDistance;

                    // アイテムスプリッタの設定
                    this.splitItem.SplitterDistance = config.ItemSplitterDistance;
                }

                // ========== ショートカット設定 ==========
                foreach (ShortCutKey shortCut in this.drawView.ShortCut.CommandKeyList)
                {
                    switch (shortCut.ShortCutCommand)
                    {
                        case ShortCutCommand.Cut:
                            this.SetMenuShortCutKey(this.menuCut, shortCut);
                            break;

                        case ShortCutCommand.Copy:
                            this.SetMenuShortCutKey(this.menuCopy, shortCut);
                            break;

                        case ShortCutCommand.Paste:
                            this.SetMenuShortCutKey(this.menuPaste, shortCut);
                            break;

                        case ShortCutCommand.UnDo:
                            this.SetMenuShortCutKey(this.menuUndo, shortCut);
                            break;

                        case ShortCutCommand.ReDo:
                            this.SetMenuShortCutKey(this.menuRedo, shortCut);
                            break;
                    }
                }
            }
            finally
            {
                // イベント処理を行うようにフラグを設定
                this.eventFlag = true;
            }
        }

        //=====================================================================
        /// <summary>
        /// メニューのショートカット設定します。
        /// </summary>
        /// <param name="item">メニューアイテム</param>
        /// <param name="shortcutKey">ショートカットキー</param>
        private void SetMenuShortCutKey(ToolStripMenuItem item, ShortCutKey shortcutKey)
        {
            Keys menuKey = Keys.None;

            if (shortcutKey != null && shortcutKey.Enabled)
                menuKey = shortcutKey.GetShortCutKeys();

            item.ShortcutKeys = menuKey;
        }

        #endregion

        #region 新規ドキュメント作成

        //=====================================================================
        /// <summary>
        /// ドキュメントの新規作成を行います。
        /// </summary>
        private void NewDoc()
        {
            // 内容が変更されている場合
            if (this.drawView.IsChanged)
            {
                // 保存確認
                DialogResult result = MessageBox.Show(
                    this,
                    MessageDef.GetMessage(MessageDef.I001001),
                    Program.PROGRAM_TITLE,
                    MessageBoxButtons.YesNoCancel,
                    MessageBoxIcon.Question);

                // キャンセル
                if (result == DialogResult.Cancel)
                    return;

                // 保存
                if (result == DialogResult.Yes)
                {
                    if (!this.Save())
                        return;
                }
            }

            // 新規ドキュメント
            DrawPanel panel = new DrawPanel();

            // 用紙サイズはＡ４
            panel.PanelSize = Program.Config.DefaultPanelSize;

            // 新しいドキュメントを作成
            this.drawView.NewDocument(panel);

            // ロードファイル名クリア
            this.loadFileName = string.Empty;

            // 更新フラグ設定
            this.drawView.IsChanged = false;

            // タイトル設定
            this.SetTitle();

            // アイテムリスト更新
            this.SetItemList();

            // ヒストリリスト更新
            this.SetHistoryList();

            // プロパティグリッド設定
            this.propertyItem.SelectedObject = this.drawView.Panel;
        }

        #endregion

        #region 上書き保存

        //=====================================================================
        /// <summary>
        /// 上書き保存を行います。
        /// </summary>
        /// <returns>キャンセルされた場合 false</returns>
        private bool Save()
        {
            bool ret = true;

            if (File.Exists(loadFileName))
            {
                try
                {
                    // マウスカーソル設定
                    this.Cursor = Cursors.WaitCursor;

                    // ファイル保存
                    using (FileStream st = new FileStream(loadFileName, FileMode.Create, FileAccess.Write))
                    {
                        this.drawView.Save(st, loadFileFormat);
                    }

                    // 更新フラグ設定
                    this.drawView.IsChanged = false;

                    // ファイル履歴設定
                    this.SetFileHistory(loadFileName);

                    // ファイル履歴メニュー設定処理
                    this.SetFileHistMenu();

                }
                catch (Exception ex)
                {
                    // エラーログ出力
                    LogWriter.WriteExceptionLog(ex);

                    // エラーメッセージ表示
                    MessageBox.Show(
                        this,
                        MessageDef.GetMessage(
                            MessageDef.E001001,
                            ex.ToString()),
                        Program.PROGRAM_TITLE,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
                finally
                {
                    // マウスカーソルを戻す
                    this.Cursor = Cursors.Default;
                }
            }
            else
            {
                // ファイル名が指定されていない場合は名前を付けて保存
                ret = SaveAs();
            }

            return ret;
        }

        #endregion

        #region 名前を付けて保存

        //=====================================================================
        /// <summary>
        /// 名前を付けて保存を行います。
        /// </summary>
        /// <returns>キャンセルされた場合 false</returns>
        private bool SaveAs()
        {
            bool ret = false;

            using (SaveFileDialog dlog = new SaveFileDialog())
            {
                // ダイアログ設定
                dlog.Filter = "J.S Draw 形式ファイル|*.jsd|ＸＭＬ形式ファイル|*.xml";
                dlog.FileName = string.IsNullOrEmpty(loadFileName) ? string.Empty : Path.GetFileNameWithoutExtension(loadFileName);
                dlog.OverwritePrompt = true;
                dlog.CheckPathExists = true;

                // フィルターインデックス設定
                if (!string.IsNullOrEmpty(loadFileName))
                {
                    switch (Path.GetExtension(loadFileName))
                    {
                        case ".jsd":
                            // Jsl 形式
                            dlog.FilterIndex = 1;
                            break;

                        case ".xml":
                            // Xml 形式
                            dlog.FilterIndex = 2;
                            break;

                        default:
                            // Jsl 形式
                            dlog.FilterIndex = 1;
                            break;
                    }
                }

                if (dlog.ShowDialog(this) == DialogResult.OK)
                {
                    FormatType fmt = FormatType.jsl;

                    // フォーマットタイプ
                    switch (dlog.FilterIndex)
                    {
                        case 1:
                            // Jsl 形式
                            fmt = FormatType.jsl;
                            break;

                        case 2:
                            // Xml 形式
                            fmt = FormatType.xml;
                            break;
                    }

                    try
                    {
                        // マウスカーソル設定
                        this.Cursor = Cursors.WaitCursor;

                        // ファイル保存
                        using (FileStream st = new FileStream(dlog.FileName, FileMode.Create, FileAccess.Write))
                        {
                            this.drawView.Save(st, fmt);
                        }

                        // ロードファイル名設定
                        loadFileName = dlog.FileName;

                        // ロードファイルフォーマット設定
                        loadFileFormat = fmt;

                        // 更新フラグ設定
                        this.drawView.IsChanged = false;

                        // タイトル設定
                        this.SetTitle();

                        // ファイル履歴設定
                        this.SetFileHistory(dlog.FileName);

                        // ファイル履歴メニュー設定処理
                        this.SetFileHistMenu();

                        ret = true;
                    }
                    catch (Exception ex)
                    {
                        // エラーログ出力
                        LogWriter.WriteExceptionLog(ex);

                        // エラーメッセージ表示
                        MessageBox.Show(
                            this,
                            MessageDef.GetMessage(
                                MessageDef.E001001,
                                ex.ToString()),
                            Program.PROGRAM_TITLE,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error);
                    }
                    finally
                    {
                        // マウスカーソルを戻す
                        this.Cursor = Cursors.Default;
                    }
                }
            }

            return ret;
        }

        #endregion

        #region 読み込み

        //=====================================================================
        /// <summary>
        /// ファイルダイアログを表示してドキュメントの読み込みを行います。
        /// </summary>
        private void Read()
        {
            using (OpenFileDialog dlog = new OpenFileDialog())
            {
                // ダイアログ設定
                dlog.Filter = "J.S Print2 形式ファイル|*.jsd|ＸＭＬ形式ファイル|*.xml";
                dlog.FileName = loadFileName;
                dlog.CheckFileExists = true;
                dlog.CheckPathExists = true;

                if (dlog.ShowDialog(this) == DialogResult.OK)
                {
                    // ファイル読み込み
                    this.Read(dlog.FileName);
                }
            }
        }


        //=====================================================================
        /// <summary>
        /// 指定されたファイルの読み込みを行います。
        /// </summary>
        /// <param name="fileName">読み込みファイル名</param>
        private void Read(string fileName)
        {
            // 内容が変更されている場合
            if (this.drawView.IsChanged)
            {
                // 保存確認
                DialogResult result = MessageBox.Show(
                    this,
                    MessageDef.GetMessage(MessageDef.I001001),
                    Program.PROGRAM_TITLE,
                    MessageBoxButtons.YesNoCancel,
                    MessageBoxIcon.Question);

                // キャンセル
                if (result == DialogResult.Cancel)
                    return;

                // 保存
                if (result == DialogResult.Yes)
                {
                    if (!this.Save())
                        return;
                }
            }

            try
            {
                // マウスカーソル設定
                this.Cursor = Cursors.WaitCursor;

                FormatType fmt = FormatType.jsl;

                // フォーマットタイプ
                switch (Path.GetExtension(fileName).ToLower())
                {
                    case ".jsd":
                        // Jsl 形式
                        fmt = FormatType.jsl;
                        break;

                    case ".xml":
                        // Xml 形式
                        fmt = FormatType.xml;
                        break;
                }

                // ファイル読み込み
                using (FileStream st = new FileStream(fileName, FileMode.Open, FileAccess.Read))
                {
                    this.drawView.Read(st, fmt);
                }

                // ロードファイル名設定
                loadFileName = fileName;

                // ロードーファイルフォーマット設定
                loadFileFormat = fmt;

                // 更新フラグ設定
                this.drawView.IsChanged = false;

                // タイトル設定
                this.SetTitle();

                // アイテムリスト更新
                this.SetItemList();

                // ヒストリリスト更新
                this.SetHistoryList();

                // プロパティグリッド設定
                this.propertyItem.SelectedObject = this.drawView.Panel;
            }
            catch (Exception ex)
            {
                // エラーログ出力
                LogWriter.WriteExceptionLog(ex);

                // エラーメッセージ表示
                MessageBox.Show(
                    this,
                    MessageDef.GetMessage(
                        MessageDef.E001002,
                        ex.ToString()),
                    Program.PROGRAM_TITLE,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            finally
            {
                // マウスカーソルを戻す
                this.Cursor = Cursors.Default;
            }
        }

        #endregion

        #region ファイル履歴設定

        //=====================================================================
        /// <summary>
        /// ファイル履歴設の理を行います。
        /// </summary>
        /// <param name="fileName">ファイル名</param>
        private void SetFileHistory(string fileName)
        {
            int index = -1;

            // すでに登録されていないかチェックする
            for (int i = 0; i < Program.Config.FileHistory.Count; i++)
            {
                if (fileName == Program.Config.FileHistory[i])
                {
                    index = i;
                    break;
                }
            }

            // すでに登録されている場合、削除する
            if (0 <= index)
                Program.Config.FileHistory.RemoveAt(index);

            // リストの先頭に登録する
            Program.Config.FileHistory.Insert(0, fileName);

            // ５件を超える場合削除
            int cnt = Program.Config.FileHistory.Count - 5;
            for (int i = 0; i < cnt; i++)
                Program.Config.FileHistory.RemoveAt(Program.Config.FileHistory.Count - 1);

            // 設定ファイル保存
            Program.WriteSettings();
        }

        #endregion

        #region アイテムリスト設定

        //=====================================================================
        /// <summary>
        /// アイテムリストの設定を行います。
        /// </summary>
        private void SetItemList()
        {
            // アイテム一覧タブが表示されていない場合は終了
            if (this.tabOperation.SelectedIndex != 0)
                return;

            // アイテム数が０の場合はリストクリア
            if (this.drawView.Items.Count == 0)
            {
                this.listItem.Items.Clear();
                return;
            }

            try
            {
                this.listItem.BeginUpdate();

                // アイテムリストの選択変更イベントを処理しないようにする
                this.eventFlag = false;

                // リストクリア
                this.listItem.Items.Clear();

                // アイテムリスト追加
                this.AddItemList(this.drawView.Items);
            }
            finally
            {
                this.listItem.EndUpdate();

                // イベントフラグを戻す
                this.eventFlag = true;
            }
        }

        //=====================================================================
        /// <summary>
        /// アイテムリスト追加を行います。
        /// </summary>
        /// <param name="collection">追加するアイテムのコレクション</param>
        private void AddItemList(DrawItemCollection collection)
        {
            IEnumerable<IDrawItem> items = 
                from IDrawItem n in collection
                orderby collection.IndexOf(n) descending
                select n;

            ListViewItem topItem = null;

            foreach (IDrawItem item in items)
            {
                // リストビューアイテム
                ListViewItem listItem = new ListViewItem();
                listItem.Tag = item;

                // アイテムイメージ
                listItem.ImageIndex = this.GetItemImageIndex(item);

                // アイテム名
                listItem.SubItems.Add(this.GetItemIndexTree(item) + item.Name);

                // アイテム位置
                listItem.SubItems.Add(
                    string.Format("{0,4:0},{1,4:0}",
                    item.Location.X, item.Location.Y));

                // サブアイテム追加
                listItem.SubItems.Add(string.Empty);

                // 選択されている場合
                if (item.IsSelected)
                {
                    listItem.Selected = true;
                    if (topItem == null)
                        topItem = listItem;
                }

                // アイテム追加
                this.listItem.Items.Add(listItem);

                // グループアイテムの場合、再帰呼び出し
                if (item is GroupItem)
                {
                    GroupItem g = item as GroupItem;
                    if (0 < g.Items.Count)
                        this.AddItemList(g.Items);
                }
            }

            this.listItem.TopItem = topItem;
        }

        //=====================================================================
        /// <summary>
        /// アイテムリストの更新を行います。
        /// </summary>
        public void UpdateItemList()
        {
            // アイテム一覧タブが表示されていない場合は終了
            if (this.tabOperation.SelectedIndex != 0)
                return;

            try
            {
                this.listItem.BeginUpdate();

                // アイテムリストの選択変更イベントを処理しないようにする
                this.eventFlag = false;
                ListViewItem topItem = null;

                foreach (ListViewItem listItem in this.listItem.Items)
                {
                    IDrawItem item = (IDrawItem)listItem.Tag;

                    // アイテムイメージ
                    listItem.ImageIndex = this.GetItemImageIndex(item);

                    if (listItem.Selected && topItem == null)
                        topItem = listItem;

                    // アイテム名
                    listItem.SubItems[1].Text = this.GetItemIndexTree(item) + item.Name;

                    // アイテム位置
                    listItem.SubItems[2].Text =
                        string.Format("{0,4:0},{1,4:0}",
                        item.Location.X, item.Location.Y);
                }

                this.listItem.TopItem = topItem;
            }
            finally
            {
                this.listItem.EndUpdate();

                // イベントフラグを戻す
                this.eventFlag = true;
            }
        }

        //=====================================================================
        /// <summary>
        /// アイテムリスト選択状態の更新を行います。
        /// </summary>
        public void UpdateItemListSelect()
        {
            // アイテム一覧タブが表示されていない場合は終了
            if (this.tabOperation.SelectedIndex != 0)
                return;

            try
            {
                this.listItem.BeginUpdate();

                // アイテムリストの選択変更イベントを処理しないようにする
                this.eventFlag = false;
                ListViewItem topItem = null;

                foreach (ListViewItem listItem in this.listItem.Items)
                {
                    // 選択されている場合
                    if (this.drawView.SelectedItems.Contains(listItem.Tag as IDrawItem))
                    {
                        listItem.Selected = true;
                        if( topItem == null)
                            topItem = listItem;
                    }
                    else
                    {
                        listItem.Selected = false;
                    }
                }

                this.listItem.TopItem = topItem;
            }
            finally
            {
                this.listItem.EndUpdate();

                // イベントフラグを戻す
                this.eventFlag = true;
            }
        }

        //=====================================================================
        /// <summary>
        /// アイテムイメージインデックスを取得します。
        /// </summary>
        /// <param name="item">インデックスを取得するアイテム</param>
        /// <returns>アイテムインデックス</returns>
        private int GetItemImageIndex(IDrawItem item)
        {
            int imageIndex = 0;

            // イメージがロックされている場合は開始インデックスを変更する。
            if (item.InheritanceDataSet.GetLock())
                imageIndex = 9;

            if (item is LineItem)
            {
                // 直線アイテム
                imageIndex += 0;
            }
            else if (item is PolyLineItem)
            {
                // 曲線アイテム
                imageIndex += 1;
            }
            else if (item is FreeLineItem)
            {
                // フリー
                imageIndex += 2;
            }
            else if (item is EllipseItem)
            {
                // 円
                imageIndex += 3;
            }
            else if (item is TrianguleItem)
            {
                // 三角形
                imageIndex += 4;
            }
            else if (item is RectItem)
            {
                // 四角アイテム
                imageIndex += 5;
            }
            else if (item is RoundItem)
            {
                // ラウンドアイテム
                imageIndex += 6;
            }
            else if (item is PolygonItem)
            {
                // 多角形アイテム
                imageIndex += 7;
            }
            else if (item is GroupItem)
            {
                // グループアイテム
                imageIndex += 8;
            }

            return imageIndex;
        }
        
        //=====================================================================
        /// <summary>
        /// アイテムインデックスのツリーを取得します。
        /// </summary>
        /// <param name="item">ツリーを取得するアイテム</param>
        /// <returns></returns>
        public string GetItemIndexTree(IDrawItem item)
        {
            // アイテムのインデックスを取得する。
            DrawItemIndex index = this.drawView.Items.ItemIndexOf(item);

            // 深度が０の場合は終了
            if (index.Depth == 0)
                return string.Empty;

            StringBuilder sb = new StringBuilder();

            for (int i = 1; i <= index.Depth; i++)
            {
                if (i == index.Depth)
                {
                    if (index[i] == 0)
                    {
                        sb.Append("└ ");
                    }
                    else
                    {
                        sb.Append("├ ");
                    }
                }
                else
                {
                    if (index[i] == 0)
                    {
                        sb.Append("　 ");
                    }
                    else
                    {
                        sb.Append("│ ");
                    }
                }
            }

            return sb.ToString();
        }

        //=====================================================================
        /// <summary>
        /// プロパティグリッド更新
        /// </summary>
        private void UpdatePropertyGrid()
        {
            // プロパティグリッドリフレッシュ
            if (this.tabOperation.SelectedIndex == 0)
                this.propertyItem.Refresh();
        }
        
        #endregion

        #region 操作履歴設定

        //=====================================================================
        /// <summary>
        /// 操作履歴リストの設定を行います。
        /// </summary>
        private void SetHistoryList()
        {
            // アイテム一覧タブが表示されていない場合は終了
            if (this.tabOperation.SelectedIndex != 1)
                return;

            // 操作履歴データ取得
            CommandHistoryData[] histData = 
                this.drawView.CommandHistory.HistoryDataList.ToArray();
            Array.Reverse(histData);

            // 操作履歴数が０の場合はリストクリア
            if (histData == null || histData.Length == 0)
            {
                this.listHistory.Items.Clear();
                this.textDetail.Text = string.Empty;
                this.toolHistoryCount.Text = "履歴件数：0";
                return;
            }

            try
            {
                this.listHistory.BeginUpdate();

                int selectedIndex = -1;
                if (this.listHistory.SelectedItems != null && 0 < this.listHistory.SelectedItems.Count)
                    selectedIndex = this.listHistory.Items.IndexOf(this.listHistory.SelectedItems[0]);

                // リストクリア
                this.listHistory.Items.Clear();

                // ========== 操作履歴リスト追加 ==========
                int index = histData.Length-1;
                int histIndex = this.drawView.CommandHistory.CurrentIndex;
                foreach( CommandHistoryData data in histData)
                {
                    // リストビューアイテム
                    ListViewItem listItem = new ListViewItem(string.Empty, index == histIndex ? 0 : -1);
                    listItem.Tag = data;

                    listItem.SubItems.Add(data.CommandName);
                    listItem.SubItems.Add(data.ComanndDataList.Count.ToString());
                    this.listHistory.Items.Add(listItem);
                    index--;
                }

                // 行選択
                if (0 <= selectedIndex && index < this.listHistory.Items.Count)
                    this.listHistory.Items[selectedIndex].Selected = true;

                this.toolHistoryCount.Text = string.Format("履歴件数：{0}", histData.Length);
            }
            finally
            {
                this.listHistory.EndUpdate();
            }

            // 履歴詳細設定
            this.SetHistoryDetail();
        }

        //=====================================================================
        /// <summary>
        /// 履歴詳細を設定します。
        /// </summary>
        private void SetHistoryDetail()
        {
            StringBuilder sb = new StringBuilder();

            if (0 < this.listHistory.SelectedItems.Count)
            {
                // 選択アイテム取得
                ListViewItem item = this.listHistory.SelectedItems[0];

                // コマンドデータ
                CommandHistoryData histData = item.Tag as CommandHistoryData;

                sb.AppendLine(string.Format("CommandName :{0}", histData.CommandName));
                sb.AppendLine(string.Format("ControlMode :{0}", histData.ControlMode.ToString()));
                sb.AppendLine(string.Format("SelectedItem:{0}",
                    histData.SelectedItemList == null ? 0 : histData.SelectedItemList.Length));
                sb.AppendLine(string.Format("CommandCount:{0}",
                    histData.ComanndDataList == null ? 0 : histData.ComanndDataList.Count));

                sb.AppendLine("");

                int num = 1;
                foreach (CommandData data in histData.ComanndDataList)
                    sb.AppendLine(string.Format("{0}.{1}", num++, data.ToString()));
            }

            this.textDetail.Text = sb.ToString();
        }

        //=====================================================================
        /// <summary>
        /// 履歴リスト選択インデックス変更イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listHistory_SelectedIndexChanged(object sender, EventArgs e)
        {
            // 履歴詳細設定
            this.SetHistoryDetail();
        }

        //=====================================================================
        /// <summary>
        /// 詳細テキストボックスダブルクリックイベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void textDetail_DoubleClick(object sender, EventArgs e)
        {
#if DEBUG
            // 詳細テキストの内容をクリップボードにコピーする。
            Clipboard.SetDataObject(this.textDetail.Text, false);
#endif
        }

        #endregion

        #region アイテムリスト[Del] キー押下イベント

        //=====================================================================
        /// <summary>
        /// アイテムリストの[Del] キーイベント押下イベントを処理します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listItem_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                // 選択されているアイテムを削除
                this.drawView.RemoveSelectedItem();
            }
        }

        #endregion

    }
}
