﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using System.Reflection;
using System.Xml.Serialization;

namespace JsDraw
{
    static class Program
    {
        /// <summary>タイトル</summary>
        public const string PROGRAM_TITLE = "J.S Draw";
        public const string PROGRAM_ID = "JsDraw";

        /// <summary>セッティングファイル名</summary>
        private static string _settingsFileName = string.Empty;

        /// <summary>設定ファイル情報</summary>
        private static ConfigInfo _config = null;

        //=====================================================================
        /// <summary>
        /// アプリケーションのメイン エントリ ポイントです。
        /// </summary>
        [STAThread]
        static void Main()
        {
            // ログクラス初期化
            LogWriter.Initialize(Path.Combine(Application.StartupPath, "Log.txt"));

            try
            {
                // 例外イベントのハンドリング
                Application.ThreadException +=
                    new System.Threading.ThreadExceptionEventHandler(Application_ThreadException);

                Thread.GetDomain().UnhandledException +=
                    new UnhandledExceptionEventHandler(Application_UnhandledException);

                // 設定ファイル名
                _settingsFileName = Path.Combine(Application.StartupPath, "config.xml");

                // 設定ファイル読み込み
                ReadSettings();

                Application.EnableVisualStyles();
                Application.SetCompatibleTextRenderingDefault(false);

                // メイン画面の表示
                MainForm frm = new MainForm();
                frm.SettingForm(_config, true);
                Application.Run(frm);

                // 設定ファイル書き込み
                WriteSettings();
            }
            catch (Exception ex)
            {
                // 例外ログ出力
                LogWriter.WriteExceptionLog(ex);

                // エラーメッセージ表示
                MessageBox.Show(
                    ex.ToString(),
                    PROGRAM_TITLE,
                    MessageBoxButtons.OK, 
                    MessageBoxIcon.Stop);
            }
        }

        //=====================================================================
        /// <summary>
        /// 設定情報
        /// </summary>
        public static ConfigInfo Config
        {
            get { return _config; }
            set { _config = value; }
        }

        //=====================================================================
        /// <summary>
        /// アプリケーション例外処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        static void Application_UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            // 例外ログ出力
            if (e.ExceptionObject is Exception)
            {
                LogWriter.WriteExceptionLog(e.ExceptionObject as Exception);
            }

            // エラーメッセージ表示
            MessageBox.Show(
                e.ExceptionObject.ToString(),
                PROGRAM_TITLE,
                MessageBoxButtons.OK,
                MessageBoxIcon.Stop);
        }

        //=====================================================================
        /// <summary>
        /// スレッド例外処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        static void Application_ThreadException(object sender, System.Threading.ThreadExceptionEventArgs e)
        {
            // 例外ログ出力
            LogWriter.WriteExceptionLog(e.Exception);

            // エラーメッセージ表示
            MessageBox.Show(
                e.Exception.ToString(),
                PROGRAM_TITLE,
                MessageBoxButtons.OK,
                MessageBoxIcon.Stop);
        }

        //=====================================================================
        /// <summary>
        /// 設定ファイル読み込み
        /// </summary>
        public static void ReadSettings()
        {
            if (File.Exists(_settingsFileName))
            {
                XmlSerializer xs = new XmlSerializer(typeof(ConfigInfo));

                using (FileStream st = new FileStream(_settingsFileName, FileMode.Open, FileAccess.Read))
                {
                    _config = (ConfigInfo)xs.Deserialize(st);

                    st.Close();
                    st.Dispose();
                }
            }
            else
            {
                // 設定ファイルが存在しない場合
                _config = new ConfigInfo();
            }
        }

        //=====================================================================
        /// <summary>
        /// 設定ファイル書き込み
        /// </summary>
        public static void WriteSettings()
        {
            if (_config != null)
            {
                XmlSerializer xs = new XmlSerializer(typeof(ConfigInfo));

                using (FileStream st = new FileStream(_settingsFileName, FileMode.Create, FileAccess.Write))
                {
                    xs.Serialize(st, _config);

                    st.Close();
                    st.Dispose();
                }
            }
        }
    }
}
